@@ -1,5 +1,5 @@
 /*
- # $Id: fortify.c,v 1.13 1998/02/08 12:42:14 fbm Exp fbm $
+ # $Id: fortify.c,v 1.16 1998/04/10 10:27:56 fbm Exp fbm $
  # Copyright (C) 1997,1998 Farrell McKay
  # All rights reserved.
  #
@@ -35,17 +35,27 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/stat.h>
+
 #include "misc.h"
 #include "index.h"
 #include "morpher.h"
+#include "os2lx.h"
+#include "trace.h"
 
-#ifndef GNU_WIN32
-#define PATH_DELIM	':'
-#define NS_PROG		"netscape"
-#else
+#if defined(GNU_WIN32)
 #define PATH_DELIM	';'
 #define NS_PROG		"netscape.exe"
+#define EXAMPLE_PATH	"C:\\Program Files\\Netscape\\Program\\netscape.exe"
+#elif defined(OS2)
+#define PATH_DELIM	';'
+#define NS_PROG		"netscape.exe"
+#define EXAMPLE_PATH	"C:\\Netscape\\netscape.exe"
+#else
+#define PATH_DELIM	':'
+#define NS_PROG		"netscape"
+#define EXAMPLE_PATH	"/usr/local/bin/netscape"
 #endif
 
 /* Unfortunately not all unistd.h headers contain this constant. */
@@ -58,7 +68,7 @@
 #endif
 
 static char		*ffy_vern  = "";
-static char		*ffy_ident = "@(#) morpher 1.6";
+static char		*ffy_ident = "@(#) morpher 1.9";
 
 typedef struct {
 	int	n;
@@ -112,12 +122,24 @@ get_path(int i)
 	int		len;
 
 	target[0] = '\0';
-	printf("Enter the path name of your %s program, or press <Rtn> to quit: ",
-		NS_PROG);
-	fgets(target, sizeof(target), stdin);
-	len = strlen(target);
-	if (len > 0 && target[len-1] == '\n')
-		target[len-1] = '\0';
+	for (;;) {
+		printf("Enter the path name of your %s program, 'h' for help, or press <Rtn> to quit: ",
+			NS_PROG);
+		fgets(target, sizeof(target), stdin);
+		len = strlen(target);
+		if (len > 0 && target[len-1] == '\n')
+			target[len-1] = '\0';
+		if (strcmp(target, "h") != 0)
+			break;
+
+		printf("\n");
+		printf("Enter the full path name of the file that you wish to fortify, for example:\n");
+		printf("    %s\n", EXAMPLE_PATH);
+		printf("\n");
+		printf("Note that you must not be running that copy of Netscape at the moment,\n");
+		printf("and you must have write permission on that file.\n");
+		printf("\n");
+	}
 	return target;
 }
 
@@ -135,7 +157,7 @@ get_target(int n, dirlist_t *dirlist)
 	for (i = 1; i < dirlist->n; i++)
 		printf("    %d = fortify \"%s\"\n", i, dirlist->paths[i-1]);
 
-	printf("    e = enter some other full path name\n", NS_PROG);
+	printf("    e = enter some other full path name\n");
 	printf("<Rtn> = quit\n");
 
 	ans[0] = '\0';
@@ -163,26 +185,55 @@ get_target(int n, dirlist_t *dirlist)
 static int
 confirm(char *action, char *tgt)
 {
-	char	buf[8];
+        char    buf[8];
+
+        printf("\n");
+
+        for (;;) {
+                printf("Do you wish to %s \"%s\" [yes|no|quit] ? ", action, tgt);
+                fgets(buf, sizeof(buf), stdin);
+                if (buf[0] == 'q' || buf[0] == 'Q')
+                        exit(0);
+                if (buf[0] == 'n' || buf[0] == 'N')
+                        return 0;
+                if (buf[0] == 'y' || buf[0] == 'Y')
+                        return 1;
+        }
+}
+
+static int
+perform_fortify(char *tgt)
+{
+	char	buf[32];
 
 	printf("\n");
 
 	for (;;) {
-		printf("Do you wish to %s \"%s\" [yes|no|quit] ? ", action, tgt);
+		printf("Please select an action [fortify|de-fortify|help|quit] : ");
 		fgets(buf, sizeof(buf), stdin);
 		if (buf[0] == 'q' || buf[0] == 'Q')
 			exit(0);
-		if (buf[0] == 'n' || buf[0] == 'N')
-			return 0;
-		if (buf[0] == 'y' || buf[0] == 'Y')
+		if (buf[0] == 'f' || buf[0] == 'F')
 			return 1;
+		if (buf[0] == 'd' || buf[0] == 'D')
+			return 0;
+		if (buf[0] == 'h' || buf[0] == 'H') {
+			printf("\n");
+			printf("Your Netscape browser currently has fortified-SSL capabilities only.\n");
+			printf("At this point, you can choose to either de-fortify your web browser,\n");
+			printf("or install all the additional strong encryption features that are provided in this\n");
+			printf("version of Fortify (for example, strong e-mail encryption and 1024-bit RSA keys).\n");
+			printf("\n");
+		}
 	}
 }
 
 static void
 cp(char *from, char *to)
 {
-	int		ifd, ofd, nb, nr, nw;
+	int		ifd = 0;
+	int		ofd = 0;
+	int		nb, nr, nw;
 	int		dot_ival = 512 * 1024;
 	char		*p, buf[16*1024];
 
@@ -236,7 +287,7 @@ static void
 dobackup(char *tgt)
 {
 	char	*bak, buf[8];
-	int	len, rtn;
+	int	len;
 
 	for (;;) {
 		printf("Do you wish to keep a backup copy of \"%s\" [yes|no|quit] ? ", tgt);
@@ -268,9 +319,10 @@ static int
 morph(char *tgt, char *morphs, int nowrite, int verbose)
 {
 	int		err;
-	int		unmorph = 0;
+	int		new_grade;
 	char		dflt_morphs[1024];
 	char		*prod;
+	char		*action;
 	index_entry_t	*ent;
 
 	if (access(tgt, W_OK) == -1) {
@@ -284,11 +336,26 @@ morph(char *tgt, char *morphs, int nowrite, int verbose)
 	printf("\n\"%s\" is .... ", tgt);
 	fflush(stdout);
 
-	ent = index_lookup(tgt);
+	ent = index_lookup(tgt, &err);
 	if (ent == NULL) {
-		printf("not recognisable.\n");
-		printf("It is either not a copy of Netscape, or it is a version\n");
-		printf("of Netscape that is not listed in the Index file.\n");
+		if (err == ERR_OPEN) {
+			printf("not recognisable.\n");
+			printf("Cannot open \"%s\": ", tgt);
+			perror("");
+		}
+		else if (err == ERR_LXCOMPR) {
+			printf("an OS/2 executable file in compressed (EXEPACK:2) format.\n");
+			printf("Please re-run Fortify after decompressing this file.\n");
+			printf("The command:\n");
+			printf("        repack.exe /exepack:0 %s\n", tgt);
+			printf("can be used to decompress the file.  Please refer to\n");
+			printf("the Fortify README documentation for further details.\n");
+		}
+		else {
+			printf("not recognisable.\n");
+			printf("It is either not a copy of Netscape, or it is a version\n");
+			printf("of Netscape that is not listed in the Index file.\n");
+		}
 		return 1;
 	}
 
@@ -306,7 +373,7 @@ morph(char *tgt, char *morphs, int nowrite, int verbose)
 		return 1;
 	}
 
-	printf("\n.... Netscape %s %s for %s (%s grade) %s\n",
+	printf("\n.... Netscape %s %s for %s (%s) %s\n",
 		prod, ent->flds[IDX_VERN], ent->flds[IDX_ARCH],
 		ent->flds[IDX_GRADE], ent->flds[IDX_COMMENTS]);
 
@@ -316,18 +383,30 @@ morph(char *tgt, char *morphs, int nowrite, int verbose)
 		return 1;
 	}
 
-	if (ent->ismorphed) {
+	if (ent->grade == ent->max_grade) {
 		if (!nowrite && confirm("de-fortify", tgt) == 0) {
 			printf("As you wish.  \"%s\" not modified.\n", tgt);
 			return 0;
 		}
-		unmorph = 1;
+		new_grade = 0;
+		action = "de-fortified";
 	}
-	else {
+	else if (ent->grade == 0) {
 		if (!nowrite && confirm("fortify", tgt) == 0) {
 			printf("As you wish.  \"%s\" not modified.\n", tgt);
 			return 0;
 		}
+		new_grade = ent->max_grade;
+		action = "fortified";
+	}
+	else {
+		if (!perform_fortify(tgt)) {
+			new_grade = 0;
+			action = "de-fortified";
+		} else {
+			new_grade = ent->max_grade;
+			action = "upgraded";
+		}
 	}
 
 	if (!nowrite)
@@ -341,11 +420,20 @@ morph(char *tgt, char *morphs, int nowrite, int verbose)
 			ent->flds[IDX_VERN], ent->flds[IDX_MORPHS]);
 		morphs = dflt_morphs;
 	}
-	err = morpher(tgt, morphs, nowrite, unmorph, verbose);
-	if (!err)
-		printf("%s. \"%s\" %ssuccessfully %sfortified.\n",
-			(!nowrite && !ent->ismorphed)? "Congratulations": "Done",
-			tgt, nowrite? "can be ": "has been ", ent->ismorphed? "de-": "");
+	err = morpher(tgt, morphs, ent, nowrite, ent->grade, new_grade, verbose);
+	if (!err) {
+		printf("%s. \"%s\" %ssuccessfully %s.\n",
+			(!nowrite && (new_grade > ent->grade))? "Congratulations": "Done",
+			tgt, nowrite? "can be ": "has been ", action);
+#if defined(OS2)
+		if (!nowrite) {
+			printf("\n");
+			printf("Note: If you have used repack.exe to de-compress this file, don't forget\n");
+			printf("to update your Netscape desktop icon so that it now refers to\n");
+			printf("\"%s\"\n", tgt);
+		}
+#endif
+	}
 
 	printf("\n");
 	return err;
@@ -354,10 +442,11 @@ morph(char *tgt, char *morphs, int nowrite, int verbose)
 static void
 usage(char *prog_name)
 {
-	fprintf(stderr, "Usage: %s [-n] [-i index] [-m morphs] [-v] [-V] [target ...]\n", prog_name);
+	fprintf(stderr, "Usage: %s [-n] [-i index] [-m morphs] [-t N] [-v] [-V] [target ...]\n", prog_name);
 	fprintf(stderr, "\t-i  override the default index file \"./Index\"\n");
 	fprintf(stderr, "\t-m  override the default morphs file\n");
 	fprintf(stderr, "\t-n  no writes; check morphs integrity only\n");
+	fprintf(stderr, "\t-t  additional trace diagnostics (level N >= 2)\n");
 	fprintf(stderr, "\t-v  verbose mode\n");
 	fprintf(stderr, "\t-V  print this program's version and exit\n");
 	exit(1);
@@ -366,7 +455,7 @@ usage(char *prog_name)
 int
 main(int argc, char *argv[])
 {
-	int		i;
+	int		i, level;
 	int		rtn = 0;
 	int		nowrite = 0;
 	int		verbose = 0;
@@ -375,6 +464,8 @@ main(int argc, char *argv[])
 	char		*morphs = NULL;
 	char		*tgt;
 
+	traceinit(0);
+
 	argc--, argv++;
 	while (argc > 0 && argv[0][0] == '-') {
 		if (argv[0][1] == '-')
@@ -393,6 +484,16 @@ main(int argc, char *argv[])
 				usage(prog_name);
 			morphs = *argv;
 		}
+		else if (argv[0][1] == 't') {
+			argc--, argv++;
+			if (argc < 1)
+				usage(prog_name);
+			level = atoi(*argv);
+			if (level < 2)
+				usage(prog_name);
+			traceinit(level);
+			verbose = 1;
+		}
 		else if (argv[0][1] == 'v')
 			verbose = 1;
 		else if (argv[0][1] == 'V') {
