@@ -1,5 +1,5 @@
 /*
- # $Id: morpher.h,v 1.2 1997/08/06 14:43:29 fbm Exp fbm $
+ # $Id: morpher.h,v 1.5 1998/04/10 10:27:56 fbm Exp fbm $
  # Copyright (C) 1997,1998 Farrell McKay
  # All rights reserved.
  #
@@ -32,21 +32,31 @@
 #ifndef DEFS_H
 #define DEFS_H
 
+#include "index.h"
+
 #define	ADDR_UNKN	(unsigned long)-1
 #define GRAMMAR_MAJOR	1
-#define GRAMMAR_MINOR	1
+#define GRAMMAR_MINOR	2
+
+#define DEFAULT_GRADE	1
 
 #define	VEC_NUM_FMT	0
 #define	VEC_STR_FMT	1
 
-#ifndef GNU_WIN32
-#define OPENFL(a)	(a)
-#else
+#define ERR_OPEN	1
+#define ERR_NOMATCH	2
+#define ERR_LXCOMPR	3
+
+#if defined(GNU_WIN32) || defined(OS2)
 #define OPENFL(a)	((a)|O_BINARY)
+#else
+#define OPENFL(a)	(a)
 #endif
 
+#ifndef OS2
 #define	min(a,b)	((a) < (b)? (a): (b))
 #define	max(a,b)	((a) > (b)? (a): (b))
+#endif
 
 #define is_bool_op(a)	((a)==eq||(a)==ne||(a)==lt||(a)==le||(a)==gt||(a)==ge)
 
@@ -87,6 +97,7 @@ typedef struct {
 	unsigned long	base;
 	unsigned long	offset;
 	unsigned long	filepos;
+	int		grade;
 	int		is_context;
 	vec_t		old, new;
 	cmd_t		*cmds;
@@ -111,4 +122,6 @@ void		dump_morph(FILE *fp, morph_set *ms, morph_t *mp, void (*eval_fn)(), void *
 void		dump_init(FILE *fp, morph_set *ms);
 void		dump_set(FILE *fp, morph_set *ms, void (*eval_fn)(), void *vp);
 
+int		morpher(char *tgt, char *morph_file, index_entry_t *ent, int nowrite, int old_grade, int new_grade, int verbose);
+
 #endif
